#include "batterywidget.h"
#include <QVBoxLayout>
#include <QDBusReply>
#include <QDebug>
#include <QStyleOption>
#include <QPainter>

BatteryWidget::BatteryWidget(QPoint point, QWidget *parent) :
    QWidget(parent),
    mPoint(point)
{
//    setWindowOpacity(0);
    setAttribute(Qt::WA_TranslucentBackground);
    initUi();
    setupComponent();
}

void BatteryWidget::initUi()
{
    QVBoxLayout *Lyt = new QVBoxLayout(this);
    Lyt->setContentsMargins(16, 16, 16, 16);
    Lyt->setSpacing(8);
    mModeLabel = new QLabel(this);
    mModeLabel->setFixedHeight(24);
    mModeLabel->setStyleSheet("QLabel{font-size: 16px;font-family: NotoSansCJKsc-Bold, NotoSansCJKsc;"
                              "font-weight: bold;}");

    QHBoxLayout *Lyt_1 = new QHBoxLayout();
    Lyt_1->setSpacing(4);

    mIconBtn = new QPushButton(this);
    mIconBtn->setFixedSize(48, 32);
    mIconBtn->setStyleSheet("QPushButton{\
                            color: rgb(255, 255, 255, 255);\
                            border: none;\
                            border-radius: 4px;\
                            outline: none;\
                        }");

    mValueLabel = new QLabel(this);
    mValueLabel->setFixedSize(48, 48);

    mStatusLabel = new QLabel(this);
    mStatusLabel->setFixedHeight(36);
    mStatusLabel->setAlignment(Qt::AlignRight);

    Lyt_1->addWidget(mIconBtn);
    Lyt_1->addWidget(mValueLabel);
    Lyt_1->addStretch();
    Lyt_1->addWidget(mStatusLabel);

    Lyt->addWidget(mModeLabel);
    Lyt->addLayout(Lyt_1);
    Lyt->addStretch();
}

void BatteryWidget::setupComponent()
{
    QDBusInterface *batInterface = new QDBusInterface(
        "org.ukui.upower", "/upower/BatteryInfo", "org.ukui.upower.battery", QDBusConnection::sessionBus(), this);

    QDBusInterface *powerInterface = new QDBusInterface(
        "org.ukui.upower", "/upower", "org.ukui.upower", QDBusConnection::sessionBus(), this);

    if (batInterface->isValid()) {
        QDBusReply<QString> iconreply = batInterface->call("IconName");
        mIconBtn->setIconSize(QSize(48, 32));
        mIconBtn->setFocusPolicy(Qt::NoFocus);
        mIconBtn->setIcon(QIcon::fromTheme(iconreply));
        QDBusConnection::sessionBus().connect(
                    QString(), "/upower/BatteryInfo", "org.ukui.upower.battery", "BatteryIcon", this, SLOT(setBatteryIcon(QString)));

        QDBusReply<bool> batteryreply = powerInterface->call("OnBattery");
        mModeLabel->setText(batteryreply ? tr("BatteryMode") : tr("PowerMode"));
    }
    ed = EngineDevice::getInstance();
    onBatteryChanged(ed->engine_get_state());

    int size;
    size = ed->devices.size();
    for (int i = 0; i < size; i++) {
        DEVICE *dv;
        dv = ed->devices.at(i);

        if (dv->m_dev.kind == UP_DEVICE_KIND_LINE_POWER) {
            continue;
        }

        connect(ed, &EngineDevice::engine_signal_Battery_State, this, &BatteryWidget::onBatteryChanged);
        continue;
    }
}

void BatteryWidget::setPoint(QPoint point)
{
    mPoint = point;
}

void BatteryWidget::showEvent(QShowEvent *event)
{
    QPoint pos = QPoint(mPoint.x(),
                        mPoint.y() - 112);
    this->move(pos);
        QWidget::showEvent(event);
}

void BatteryWidget::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    p.setRenderHint(QPainter::Antialiasing, true);
    p.save();
    p.setBrush(opt.palette.color(QPalette::Base));
    p.setPen(Qt::transparent);
    p.setOpacity(0.75);
    p.drawRoundedRect(this->rect(), 16, 16);
    p.restore();
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void BatteryWidget::setBatteryIcon(QString icon)
{
    mIconBtn->setIcon(QIcon::fromTheme(icon));
}

void BatteryWidget::onBatteryChanged(QStringList args)
{
    int battery = args.at(0).toInt();
    int batteryState = args.at(1).toInt();
    if (batteryState == 1 || batteryState == 5) {
        mStatusLabel->setText(tr("Charging..."));
    } else if (batteryState == 4) {
        mStatusLabel->setText(tr("fully charged"));
    }

    if (batteryState == 4 || batteryState == 1 || batteryState == 5) {
        mStatusLabel->setVisible(true);
        mModeLabel->setText(tr("PowerMode"));
    } else {
        mStatusLabel->setVisible(false);
        mModeLabel->setText(tr("BatteryMode"));
    }
    mValueLabel->setText(QString("<font size='5';font color=#262626>%1</font>%").arg(battery));
}
