#include "surewindow.h"
#include "ui_surewindow.h"
#include <QDebug>
#include <QStandardItemModel>
SureWindow::SureWindow(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::SureWindow)
{
    ui->setupUi(this);
    ui->tipLabel->setStyleSheet("color:white;font:14pt;");
    //ui->tipLabel->setWordWrap(true);
    connect(ui->cancelBtn, &QPushButton::clicked, this, [&]() { emit cantelButtonclicked(); });
    connect(ui->confirmBtn, &QPushButton::clicked, this, [&]() { emit confirmButtonclicked(); });
}

SureWindow::~SureWindow()
{
    delete ui;
}

void SureWindow::setTips(const QString tips)
{
    ui->cancelBtn->setStyleSheet("QPushButton{background: rgba(255, 255, 255, 0.2);border-radius: 8px;}"
                                 "QPushButton:hover{background: rgba(255, 255, 255, 0.4);border-radius: 8px;}"
                                 "QPushButton:pressed {background: rgba(255, 255, 255, 0.3);border-radius: 8px;}");

    ui->confirmBtn->setStyleSheet("QPushButton{background: rgba(255, 255, 255, 0.2);border-radius: 8px;}"
                                 "QPushButton:hover{background: rgba(255, 255, 255, 0.4);border-radius: 8px;}"
                                 "QPushButton:pressed {background: rgba(255, 255, 255, 0.3);border-radius: 8px;}");
    ui->confirmBtn->show();
    ui->tipLabel->show();
    ui->listView->hide();
    ui->tipLabel->setText(tips);
}

void SureWindow::setWarning(QVector<InhibitInfo::InhibitorInfo> &list, int type)
{
    switch (type) {
    case 0:
        ui->tipLabel->setText(tr("The following program is running to prevent the system from reboot!"));
        break;
    case 1:
        ui->tipLabel->setText(tr("The following program is running to prevent the system from shutting down!"));
        break;
    case 2:
        ui->tipLabel->setText(tr("The following program is running to prevent the system from suspend!"));
        break;
    case 3:
        ui->tipLabel->setText(tr("The following program is running to prevent the system from hibernate!"));
        break;
    default:
        break;
    }
    ui->tipLabel->adjustSize();
    adjustSize();
    ui->listView->show();

    QStandardItemModel *model = new QStandardItemModel(this);
    for (auto iter = list.begin(); iter != list.end(); ++iter) {
        QIcon icon;
        QString appName = iter->name;
        QString iconName = iter->icon;

        if (!iconName.isEmpty() && QIcon::hasThemeIcon(iconName)) {
            icon = QIcon::fromTheme(iconName);
        } else if (QIcon::hasThemeIcon("application-x-desktop")) {
            icon = QIcon::fromTheme("application-x-desktop");
        }
        model->appendRow(new QStandardItem(icon, appName));
    }

    ui->listView->verticalScrollMode();
    ui->listView->setStyleSheet("font:10pt;color:white; background-color: rgba(255,255,255,30%); border-radius: 12px;");
    ui->listView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    ui->listView->setIconSize(QSize(32,32));
    ui->listView->setModel(model);
    ui->listView->setFixedSize(520, 320);
    adjustSize();
    ui->cancelBtn->setFixedSize(120, 48);

    ui->confirmBtn->hide();
    ui->cancelBtn->setStyleSheet("QPushButton{background: rgba(255, 255, 255, 0.2);border-radius: 24px;}"
                                 "QPushButton:hover{background: rgba(255, 255, 255, 0.4);border-radius: 24px;}"
                                 "QPushButton:pressed {background: rgba(255, 255, 255, 0.3);border-radius: 24px;}");

}
